@@ -10,7 +10,7 @@ our @EXPORT_OK = ( Types::Standard->type_names );
 
 our $meta = __PACKAGE__->meta;
 
-our $VERSION = '0.000002';
+our $VERSION = '0.000003';
 
 our (%entity, %recurse, $esc, $unesc, $path);
 BEGIN {
@@ -59,7 +59,7 @@ BEGIN {
 			undef; 
 		} || $meta->add_type({ 
 			name => $parent->name . $counter++,
-			parent => $parent->{abuse_parent} ? $parent->{abuse_parent}->($hide) : $parent,
+			parent => $parent->{abuse_parent} && $parent->{abuse_parent}->($hide) || $parent,
 			coercion => $parent->{abuse}->($hide),
 			abuse => $hide,
 			($parent->{abuse_constraint} ? (constraint => $parent->{abuse_constraint}->($hide)) : ())
@@ -346,10 +346,44 @@ $meta->add_type({
 	abuse => \&_html,
 	coercion => sub {
 		my $ref = ref $_[0];
-		return $ref eq 'ARRAY' ? scalar @{$_[0]} : scalar keys @{$_[0]};
+		return $ref eq 'ARRAY' ? scalar @{$_[0]} : scalar keys %{$_[0]};
 	},
 });
 
+$meta->add_type({
+	name => 'JSON',
+	parent => scalar $meta->Any,
+	constraint => sub {
+		my $ref = ref $_[0];
+		$ref ? 1 : 0;
+	},
+	coercion => sub {
+		require JSON;
+		my $json = JSON->new;
+		return $json->decode($_[0]);
+	},	
+	abuse_parent => \&_json_change,
+	abuse => \&_json
+});
+
+sub _json_change {
+	my $ref = ref $_[0];
+ 	return unless ! $ref || $ref eq 'ARRAY';
+	my $key = $ref eq 'ARRAY' ? $_[0]->[0] : $_[0];
+	my $type = eval{ $meta->$key };
+	$type ? shift @{$_[0]} : do { $type = $meta->Str if $key eq 'encode'; };
+	$type;
+}
+
+sub _json {
+	require JSON;
+	my $json = JSON->new;
+	my $ref = ref $_[0];
+	my $type = $ref ? $_[0]->[0] : $_[0];
+	map { $json = $json->$_ } @{ $_[0]->[1] } if ( $ref && ref $_[0]->[1] eq 'ARRAY' );
+	return sub {  $json->$type($_[0]) };
+}
+
 sub recurse {
 	my ($yelp, $recurse) = @_;
     my $ref = reftype($yelp);
@@ -367,47 +401,6 @@ sub _deep_recurse {
 					: $_[0];
 }
 
-=pod
-$meta->add_type({
-	name => 'Enc',
-	parent => scalar $meta->Any,
-	abuse_parent => \&_encode_change,
-	abuse_constraint => \&_encode_constraint,
-	abuse => \&_encode
-});
-sub _encode_change {
-	my ($help) = shift;
-	return $meta->Any unless ref $help;
-	my $type = $help->[0];
-	return $meta->$type;
-}
-sub _encode_constraint {
-	my $help = ref $_[0] ? $_[0]->[1] : $_[0];
-	# not saying this efficient /o\ :)
-	my %r = (
-		%recurse,
-		scalarref => sub { ${$_[0]} =~ m/^[0-9.]+$/g ? $_[0] : do { recurse(${$_[0]}, $_[1], $_[2]); $_[0] }; }, 
-		scalar => sub { my $checks = Encode::decode($help, $_[0], Encode::FB_CROAK); }
-	);
-	return sub {  
-		my $str = $_[0];
-		eval { $str = _recurse($str, \%r, {}); };
-		$@ ? 0 : 1;
-	}
-}
-sub _encode {
-	my $help = ref $_[0] ? $_[0]->[1] : $_[0];
-	my %r = (
-		%recurse,
-		scalar => sub { eval { $_[0] = Encode::encode($help, $_[0]); }; $_[0]; }
-	);
-	return sub {  
-		my $str = $_[0];
-		$str = _recurse($str, \%r, {});;
-		$str;
-	}
-}
-=cut
 
 # TODO a little documentations
 # TBC .....
@@ -424,7 +417,7 @@ Coerce::Types::Standard - Coercing
 
 =head1 VERSION
 
-Version 0.000002
+Version 0.000003
 
 =cut
 
